# -*- coding: utf-8 -*-

"""
Azerothcore World of Warcraft Database Application.

代码结构:

- orm, orm_getter: 对底层数据表以及数据库连接的封装.
- app: 数据库 app 的实现, 并将其封装成用户友好的 Python 函数.
- cli: 命令行接口, 部署在在 EC2 游戏服务器上.
- sdk: 通过 SSM Run Command 远程调用 cli 的 Python SDK.
- gui: 一个图形界面的 App.
"""

from ._version import __version__

__short_description__ = "Azerothcore World of Warcraft Database Application."
__license__ = "MIT"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__maintainer__ = "Sanhe Hu"
__maintainer_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"



# -*- coding: utf-8 -*-

import typing as T

from PySide6 import QtCore, QtWidgets, QtGui

if T.TYPE_CHECKING:
    from .main import QuestCompleterWidget


class QuestCompleterWidgetQuestAction:
    def add_quest_action(self):
        self._add_quest_action_widgets()
        self._add_quest_action_layout()

    def _add_quest_action_widgets(self):
        self.quest_action_item_list_header = QtWidgets.QLabel("Actions")
        self.quest_action_item_list_header.setAlignment(QtCore.Qt.AlignCenter)

        self.quest_action_item_list = QtWidgets.QListWidget()
        self.quest_action_item_list.setSelectionMode(
            QtWidgets.QAbstractItemView.SelectionMode.SingleSelection
        )

        self.quest_action_item_list.itemDoubleClicked.connect(
            self._quest_action_item_double_clicked_event_handler
        )

    def _quest_action_item_double_clicked_event_handler(self):
        """
        当用户双击 Quest Action 中的某个 item 的时候会触发这个函数.
        """
        selected_items = self.quest_action_item_list.selectedItems()
        item = selected_items[0]
        text = item.text()
        print(f"item list item double clicked: {text!r}")

        clipboard = QtWidgets.QApplication.clipboard()
        clipboard.setText(item.copy_text)

    def _add_quest_action_layout(self):
        self.quest_action_layout = QtWidgets.QVBoxLayout()
        self.quest_action_layout.addWidget(self.quest_action_item_list_header)
        self.quest_action_layout.addWidget(self.quest_action_item_list)



# -*- coding: utf-8 -*-

import enum


class LocaleEnum(str, enum.Enum):
    """
    Reference:

    - https://www.azerothcore.org/wiki/acore_string
    """
    enUS = "enUS"
    deDE = "deDE"
    esES = "esES"
    esMX = "esMX"
    frFR = "frFR"
    ruRU = "ruRU"
    zhCN = "zhCN"
    zhTW = "zhTW"



# -*- coding: utf-8 -*-

"""
该模块使用 Sqlalchemy 枚举了所有 azerothcore 中的数据库和数据表对象. 由于里面的表众多,
我们不可能一一的根据表名, 列名, 列类型来定义, 所以我们用了 metadata.reflect 的方式来自动
获得所有的表的 Metadata, 并将其缓存到磁盘上, 以便下次使用.

所有的数据库 App 都要使用这个模块来构造 SQL query.
"""

import pickle

import dataclasses
import sqlalchemy as sa

from .paths import (
    path_metadata_cache,
)
from .compat import cached_property


@dataclasses.dataclass
class Orm:
    """
    一个可以访问所有的数据表对象 ``sqlalchemy.Table`` 的 namespace 类.
    """

    engine: sa.engine.Engine

    _metadata: sa.MetaData = dataclasses.field(init=False, repr=False)

    def _reflect(self) -> sa.MetaData:
        metadata = sa.MetaData()
        # 注: 只有指定 schema 才能用一个 metadata 来管理多个数据库 (在 MySQL 中是
        # database, 但在数据库学术领域叫 schema, 例如 Postgres 中就是 schema)
        metadata.reflect(self.engine, schema="acore_auth")
        metadata.reflect(self.engine, schema="acore_characters")
        metadata.reflect(self.engine, schema="acore_world")
        with path_metadata_cache.open("wb") as f:
            pickle.dump(metadata, f)
        return metadata

    def __post_init__(self):
        # 如果缓存存在则从缓存读取 metadata, 否则从数据库中 reflect 出来, 再写入缓存
        if path_metadata_cache.exists():
            try:
                with path_metadata_cache.open("rb") as f:
                    self._metadata = pickle.load(f)
            except pickle.UnpicklingError:
                self._metadata = self._reflect()
        else:
            self._metadata = self._reflect()

    @cached_property
    def t_account(self) -> sa.Table:
        return self._metadata.tables["acore_auth.account"]

    @cached_property
    def t_account_access(self) -> sa.Table:
        return self._metadata.tables["acore_auth.account_access"]

    @cached_property
    def t_account_banned(self) -> sa.Table:
        return self._metadata.tables["acore_auth.account_banned"]

    @cached_property
    def t_account_muted(self) -> sa.Table:
        return self._metadata.tables["acore_auth.account_muted"]

    @cached_property
    def t_autobroadcast(self) -> sa.Table:
        return self._metadata.tables["acore_auth.autobroadcast"]

    @cached_property
    def t_build_info(self) -> sa.Table:
        return self._metadata.tables["acore_auth.build_info"]

    @cached_property
    def t_ip_banned(self) -> sa.Table:
        return self._metadata.tables["acore_auth.ip_banned"]

    @cached_property
    def t_logs(self) -> sa.Table:
        return self._metadata.tables["acore_auth.logs"]

    @cached_property
    def t_logs_ip_actions(self) -> sa.Table:
        return self._metadata.tables["acore_auth.logs_ip_actions"]

    @cached_property
    def t_motd(self) -> sa.Table:
        return self._metadata.tables["acore_auth.motd"]

    @cached_property
    def t_realmcharacters(self) -> sa.Table:
        return self._metadata.tables["acore_auth.realmcharacters"]

    @cached_property
    def t_realmlist(self) -> sa.Table:
        return self._metadata.tables["acore_auth.realmlist"]

    @cached_property
    def t_secret_digest(self) -> sa.Table:
        return self._metadata.tables["acore_auth.secret_digest"]

    @cached_property
    def t_updates(self) -> sa.Table:
        return self._metadata.tables["acore_auth.updates"]

    @cached_property
    def t_updates_include(self) -> sa.Table:
        return self._metadata.tables["acore_auth.updates_include"]

    @cached_property
    def t_uptime(self) -> sa.Table:
        return self._metadata.tables["acore_auth.uptime"]

    @cached_property
    def t_account_data(self) -> sa.Table:
        return self._metadata.tables["acore_characters.account_data"]

    @cached_property
    def t_account_instance_times(self) -> sa.Table:
        return self._metadata.tables["acore_characters.account_instance_times"]

    @cached_property
    def t_account_tutorial(self) -> sa.Table:
        return self._metadata.tables["acore_characters.account_tutorial"]

    @cached_property
    def t_addons(self) -> sa.Table:
        return self._metadata.tables["acore_characters.addons"]

    @cached_property
    def t_arena_team(self) -> sa.Table:
        return self._metadata.tables["acore_characters.arena_team"]

    @cached_property
    def t_arena_team_member(self) -> sa.Table:
        return self._metadata.tables["acore_characters.arena_team_member"]

    @cached_property
    def t_auctionhouse(self) -> sa.Table:
        return self._metadata.tables["acore_characters.auctionhouse"]

    @cached_property
    def t_banned_addons(self) -> sa.Table:
        return self._metadata.tables["acore_characters.banned_addons"]

    @cached_property
    def t_battleground_deserters(self) -> sa.Table:
        return self._metadata.tables["acore_characters.battleground_deserters"]

    @cached_property
    def t_bugreport(self) -> sa.Table:
        return self._metadata.tables["acore_characters.bugreport"]

    @cached_property
    def t_calendar_events(self) -> sa.Table:
        return self._metadata.tables["acore_characters.calendar_events"]

    @cached_property
    def t_calendar_invites(self) -> sa.Table:
        return self._metadata.tables["acore_characters.calendar_invites"]

    @cached_property
    def t_channels(self) -> sa.Table:
        return self._metadata.tables["acore_characters.channels"]

    @cached_property
    def t_channels_bans(self) -> sa.Table:
        return self._metadata.tables["acore_characters.channels_bans"]

    @cached_property
    def t_channels_rights(self) -> sa.Table:
        return self._metadata.tables["acore_characters.channels_rights"]

    @cached_property
    def t_character_account_data(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_account_data"]

    @cached_property
    def t_character_achievement(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_achievement"]

    @cached_property
    def t_character_achievement_progress(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_achievement_progress"]

    @cached_property
    def t_character_action(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_action"]

    @cached_property
    def t_character_arena_stats(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_arena_stats"]

    @cached_property
    def t_character_aura(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_aura"]

    @cached_property
    def t_character_banned(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_banned"]

    @cached_property
    def t_character_battleground_random(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_battleground_random"]

    @cached_property
    def t_character_brew_of_the_month(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_brew_of_the_month"]

    @cached_property
    def t_character_declinedname(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_declinedname"]

    @cached_property
    def t_character_entry_point(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_entry_point"]

    @cached_property
    def t_character_equipmentsets(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_equipmentsets"]

    @cached_property
    def t_character_gifts(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_gifts"]

    @cached_property
    def t_character_glyphs(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_glyphs"]

    @cached_property
    def t_character_homebind(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_homebind"]

    @cached_property
    def t_character_instance(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_instance"]

    @cached_property
    def t_character_inventory(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_inventory"]

    @cached_property
    def t_character_pet(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_pet"]

    @cached_property
    def t_character_pet_declinedname(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_pet_declinedname"]

    @cached_property
    def t_character_queststatus(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_queststatus"]

    @cached_property
    def t_character_queststatus_daily(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_queststatus_daily"]

    @cached_property
    def t_character_queststatus_monthly(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_queststatus_monthly"]

    @cached_property
    def t_character_queststatus_rewarded(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_queststatus_rewarded"]

    @cached_property
    def t_character_queststatus_seasonal(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_queststatus_seasonal"]

    @cached_property
    def t_character_queststatus_weekly(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_queststatus_weekly"]

    @cached_property
    def t_character_reputation(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_reputation"]

    @cached_property
    def t_character_settings(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_settings"]

    @cached_property
    def t_character_skills(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_skills"]

    @cached_property
    def t_character_social(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_social"]

    @cached_property
    def t_character_spell(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_spell"]

    @cached_property
    def t_character_spell_cooldown(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_spell_cooldown"]

    @cached_property
    def t_character_stats(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_stats"]

    @cached_property
    def t_character_talent(self) -> sa.Table:
        return self._metadata.tables["acore_characters.character_talent"]

    @cached_property
    def t_characters(self) -> sa.Table:
        return self._metadata.tables["acore_characters.characters"]

    @cached_property
    def t_corpse(self) -> sa.Table:
        return self._metadata.tables["acore_characters.corpse"]

    @cached_property
    def t_creature_respawn(self) -> sa.Table:
        return self._metadata.tables["acore_characters.creature_respawn"]

    @cached_property
    def t_game_event_condition_save(self) -> sa.Table:
        return self._metadata.tables["acore_characters.game_event_condition_save"]

    @cached_property
    def t_game_event_save(self) -> sa.Table:
        return self._metadata.tables["acore_characters.game_event_save"]

    @cached_property
    def t_gameobject_respawn(self) -> sa.Table:
        return self._metadata.tables["acore_characters.gameobject_respawn"]

    @cached_property
    def t_gm_subsurvey(self) -> sa.Table:
        return self._metadata.tables["acore_characters.gm_subsurvey"]

    @cached_property
    def t_gm_survey(self) -> sa.Table:
        return self._metadata.tables["acore_characters.gm_survey"]

    @cached_property
    def t_gm_ticket(self) -> sa.Table:
        return self._metadata.tables["acore_characters.gm_ticket"]

    @cached_property
    def t_group_member(self) -> sa.Table:
        return self._metadata.tables["acore_characters.group_member"]

    @cached_property
    def t_groups(self) -> sa.Table:
        return self._metadata.tables["acore_characters.groups"]

    @cached_property
    def t_guild(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild"]

    @cached_property
    def t_guild_bank_eventlog(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild_bank_eventlog"]

    @cached_property
    def t_guild_bank_item(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild_bank_item"]

    @cached_property
    def t_guild_bank_right(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild_bank_right"]

    @cached_property
    def t_guild_bank_tab(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild_bank_tab"]

    @cached_property
    def t_guild_eventlog(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild_eventlog"]

    @cached_property
    def t_guild_member(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild_member"]

    @cached_property
    def t_guild_member_withdraw(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild_member_withdraw"]

    @cached_property
    def t_guild_rank(self) -> sa.Table:
        return self._metadata.tables["acore_characters.guild_rank"]

    @cached_property
    def t_instance(self) -> sa.Table:
        return self._metadata.tables["acore_characters.instance"]

    @cached_property
    def t_instance_reset(self) -> sa.Table:
        return self._metadata.tables["acore_characters.instance_reset"]

    @cached_property
    def t_instance_saved_go_state_data(self) -> sa.Table:
        return self._metadata.tables["acore_characters.instance_saved_go_state_data"]

    @cached_property
    def t_item_instance(self) -> sa.Table:
        return self._metadata.tables["acore_characters.item_instance"]

    @cached_property
    def t_item_loot_storage(self) -> sa.Table:
        return self._metadata.tables["acore_characters.item_loot_storage"]

    @cached_property
    def t_item_refund_instance(self) -> sa.Table:
        return self._metadata.tables["acore_characters.item_refund_instance"]

    @cached_property
    def t_item_soulbound_trade_data(self) -> sa.Table:
        return self._metadata.tables["acore_characters.item_soulbound_trade_data"]

    @cached_property
    def t_lag_reports(self) -> sa.Table:
        return self._metadata.tables["acore_characters.lag_reports"]

    @cached_property
    def t_lfg_data(self) -> sa.Table:
        return self._metadata.tables["acore_characters.lfg_data"]

    @cached_property
    def t_log_arena_fights(self) -> sa.Table:
        return self._metadata.tables["acore_characters.log_arena_fights"]

    @cached_property
    def t_log_arena_memberstats(self) -> sa.Table:
        return self._metadata.tables["acore_characters.log_arena_memberstats"]

    @cached_property
    def t_log_encounter(self) -> sa.Table:
        return self._metadata.tables["acore_characters.log_encounter"]

    @cached_property
    def t_log_money(self) -> sa.Table:
        return self._metadata.tables["acore_characters.log_money"]

    @cached_property
    def t_mail(self) -> sa.Table:
        return self._metadata.tables["acore_characters.mail"]

    @cached_property
    def t_mail_items(self) -> sa.Table:
        return self._metadata.tables["acore_characters.mail_items"]

    @cached_property
    def t_mail_server_character(self) -> sa.Table:
        return self._metadata.tables["acore_characters.mail_server_character"]

    @cached_property
    def t_mail_server_template(self) -> sa.Table:
        return self._metadata.tables["acore_characters.mail_server_template"]

    @cached_property
    def t_pet_aura(self) -> sa.Table:
        return self._metadata.tables["acore_characters.pet_aura"]

    @cached_property
    def t_pet_spell(self) -> sa.Table:
        return self._metadata.tables["acore_characters.pet_spell"]

    @cached_property
    def t_pet_spell_cooldown(self) -> sa.Table:
        return self._metadata.tables["acore_characters.pet_spell_cooldown"]

    @cached_property
    def t_petition(self) -> sa.Table:
        return self._metadata.tables["acore_characters.petition"]

    @cached_property
    def t_petition_sign(self) -> sa.Table:
        return self._metadata.tables["acore_characters.petition_sign"]

    @cached_property
    def t_pool_quest_save(self) -> sa.Table:
        return self._metadata.tables["acore_characters.pool_quest_save"]

    @cached_property
    def t_profanity_name(self) -> sa.Table:
        return self._metadata.tables["acore_characters.profanity_name"]

    @cached_property
    def t_pvpstats_battlegrounds(self) -> sa.Table:
        return self._metadata.tables["acore_characters.pvpstats_battlegrounds"]

    @cached_property
    def t_pvpstats_players(self) -> sa.Table:
        return self._metadata.tables["acore_characters.pvpstats_players"]

    @cached_property
    def t_quest_tracker(self) -> sa.Table:
        return self._metadata.tables["acore_characters.quest_tracker"]

    @cached_property
    def t_recovery_item(self) -> sa.Table:
        return self._metadata.tables["acore_characters.recovery_item"]

    @cached_property
    def t_reserved_name(self) -> sa.Table:
        return self._metadata.tables["acore_characters.reserved_name"]

    @cached_property
    def t_updates(self) -> sa.Table:
        return self._metadata.tables["acore_characters.updates"]

    @cached_property
    def t_updates_include(self) -> sa.Table:
        return self._metadata.tables["acore_characters.updates_include"]

    @cached_property
    def t_warden_action(self) -> sa.Table:
        return self._metadata.tables["acore_characters.warden_action"]

    @cached_property
    def t_worldstates(self) -> sa.Table:
        return self._metadata.tables["acore_characters.worldstates"]

    @cached_property
    def t_achievement_category_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.achievement_category_dbc"]

    @cached_property
    def t_achievement_criteria_data(self) -> sa.Table:
        return self._metadata.tables["acore_world.achievement_criteria_data"]

    @cached_property
    def t_achievement_criteria_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.achievement_criteria_dbc"]

    @cached_property
    def t_achievement_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.achievement_dbc"]

    @cached_property
    def t_achievement_reward(self) -> sa.Table:
        return self._metadata.tables["acore_world.achievement_reward"]

    @cached_property
    def t_achievement_reward_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.achievement_reward_locale"]

    @cached_property
    def t_acore_string(self) -> sa.Table:
        return self._metadata.tables["acore_world.acore_string"]

    @cached_property
    def t_areagroup_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.areagroup_dbc"]

    @cached_property
    def t_areapoi_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.areapoi_dbc"]

    @cached_property
    def t_areatable_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.areatable_dbc"]

    @cached_property
    def t_areatrigger(self) -> sa.Table:
        return self._metadata.tables["acore_world.areatrigger"]

    @cached_property
    def t_areatrigger_involvedrelation(self) -> sa.Table:
        return self._metadata.tables["acore_world.areatrigger_involvedrelation"]

    @cached_property
    def t_areatrigger_scripts(self) -> sa.Table:
        return self._metadata.tables["acore_world.areatrigger_scripts"]

    @cached_property
    def t_areatrigger_tavern(self) -> sa.Table:
        return self._metadata.tables["acore_world.areatrigger_tavern"]

    @cached_property
    def t_areatrigger_teleport(self) -> sa.Table:
        return self._metadata.tables["acore_world.areatrigger_teleport"]

    @cached_property
    def t_auctionhouse_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.auctionhouse_dbc"]

    @cached_property
    def t_bankbagslotprices_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.bankbagslotprices_dbc"]

    @cached_property
    def t_barbershopstyle_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.barbershopstyle_dbc"]

    @cached_property
    def t_battleground_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.battleground_template"]

    @cached_property
    def t_battlemaster_entry(self) -> sa.Table:
        return self._metadata.tables["acore_world.battlemaster_entry"]

    @cached_property
    def t_battlemasterlist_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.battlemasterlist_dbc"]

    @cached_property
    def t_broadcast_text(self) -> sa.Table:
        return self._metadata.tables["acore_world.broadcast_text"]

    @cached_property
    def t_broadcast_text_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.broadcast_text_locale"]

    @cached_property
    def t_charstartoutfit_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.charstartoutfit_dbc"]

    @cached_property
    def t_chartitles_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.chartitles_dbc"]

    @cached_property
    def t_chatchannels_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.chatchannels_dbc"]

    @cached_property
    def t_chrclasses_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.chrclasses_dbc"]

    @cached_property
    def t_chrraces_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.chrraces_dbc"]

    @cached_property
    def t_cinematiccamera_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.cinematiccamera_dbc"]

    @cached_property
    def t_cinematicsequences_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.cinematicsequences_dbc"]

    @cached_property
    def t_command(self) -> sa.Table:
        return self._metadata.tables["acore_world.command"]

    @cached_property
    def t_conditions(self) -> sa.Table:
        return self._metadata.tables["acore_world.conditions"]

    @cached_property
    def t_creature(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature"]

    @cached_property
    def t_creature_addon(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_addon"]

    @cached_property
    def t_creature_classlevelstats(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_classlevelstats"]

    @cached_property
    def t_creature_equip_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_equip_template"]

    @cached_property
    def t_creature_formations(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_formations"]

    @cached_property
    def t_creature_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_loot_template"]

    @cached_property
    def t_creature_model_info(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_model_info"]

    @cached_property
    def t_creature_movement_override(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_movement_override"]

    @cached_property
    def t_creature_onkill_reputation(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_onkill_reputation"]

    @cached_property
    def t_creature_questender(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_questender"]

    @cached_property
    def t_creature_questitem(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_questitem"]

    @cached_property
    def t_creature_queststarter(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_queststarter"]

    @cached_property
    def t_creature_summon_groups(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_summon_groups"]

    @cached_property
    def t_creature_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_template"]

    @cached_property
    def t_creature_template_addon(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_template_addon"]

    @cached_property
    def t_creature_template_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_template_locale"]

    @cached_property
    def t_creature_template_movement(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_template_movement"]

    @cached_property
    def t_creature_template_resistance(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_template_resistance"]

    @cached_property
    def t_creature_template_spell(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_template_spell"]

    @cached_property
    def t_creature_text(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_text"]

    @cached_property
    def t_creature_text_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.creature_text_locale"]

    @cached_property
    def t_creaturedisplayinfo_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.creaturedisplayinfo_dbc"]

    @cached_property
    def t_creaturedisplayinfoextra_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.creaturedisplayinfoextra_dbc"]

    @cached_property
    def t_creaturefamily_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.creaturefamily_dbc"]

    @cached_property
    def t_creaturemodeldata_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.creaturemodeldata_dbc"]

    @cached_property
    def t_creaturespelldata_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.creaturespelldata_dbc"]

    @cached_property
    def t_creaturetype_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.creaturetype_dbc"]

    @cached_property
    def t_currencytypes_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.currencytypes_dbc"]

    @cached_property
    def t_destructiblemodeldata_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.destructiblemodeldata_dbc"]

    @cached_property
    def t_disables(self) -> sa.Table:
        return self._metadata.tables["acore_world.disables"]

    @cached_property
    def t_disenchant_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.disenchant_loot_template"]

    @cached_property
    def t_dungeon_access_requirements(self) -> sa.Table:
        return self._metadata.tables["acore_world.dungeon_access_requirements"]

    @cached_property
    def t_dungeon_access_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.dungeon_access_template"]

    @cached_property
    def t_dungeonencounter_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.dungeonencounter_dbc"]

    @cached_property
    def t_durabilitycosts_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.durabilitycosts_dbc"]

    @cached_property
    def t_durabilityquality_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.durabilityquality_dbc"]

    @cached_property
    def t_emotes_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.emotes_dbc"]

    @cached_property
    def t_emotestext_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.emotestext_dbc"]

    @cached_property
    def t_event_scripts(self) -> sa.Table:
        return self._metadata.tables["acore_world.event_scripts"]

    @cached_property
    def t_exploration_basexp(self) -> sa.Table:
        return self._metadata.tables["acore_world.exploration_basexp"]

    @cached_property
    def t_faction_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.faction_dbc"]

    @cached_property
    def t_factiontemplate_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.factiontemplate_dbc"]

    @cached_property
    def t_fishing_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.fishing_loot_template"]

    @cached_property
    def t_game_event(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event"]

    @cached_property
    def t_game_event_arena_seasons(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_arena_seasons"]

    @cached_property
    def t_game_event_battleground_holiday(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_battleground_holiday"]

    @cached_property
    def t_game_event_condition(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_condition"]

    @cached_property
    def t_game_event_creature(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_creature"]

    @cached_property
    def t_game_event_creature_quest(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_creature_quest"]

    @cached_property
    def t_game_event_gameobject(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_gameobject"]

    @cached_property
    def t_game_event_gameobject_quest(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_gameobject_quest"]

    @cached_property
    def t_game_event_model_equip(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_model_equip"]

    @cached_property
    def t_game_event_npc_vendor(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_npc_vendor"]

    @cached_property
    def t_game_event_npcflag(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_npcflag"]

    @cached_property
    def t_game_event_pool(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_pool"]

    @cached_property
    def t_game_event_prerequisite(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_prerequisite"]

    @cached_property
    def t_game_event_quest_condition(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_quest_condition"]

    @cached_property
    def t_game_event_seasonal_questrelation(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_event_seasonal_questrelation"]

    @cached_property
    def t_game_graveyard(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_graveyard"]

    @cached_property
    def t_game_tele(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_tele"]

    @cached_property
    def t_game_weather(self) -> sa.Table:
        return self._metadata.tables["acore_world.game_weather"]

    @cached_property
    def t_gameobject(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject"]

    @cached_property
    def t_gameobject_addon(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject_addon"]

    @cached_property
    def t_gameobject_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject_loot_template"]

    @cached_property
    def t_gameobject_questender(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject_questender"]

    @cached_property
    def t_gameobject_questitem(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject_questitem"]

    @cached_property
    def t_gameobject_queststarter(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject_queststarter"]

    @cached_property
    def t_gameobject_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject_template"]

    @cached_property
    def t_gameobject_template_addon(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject_template_addon"]

    @cached_property
    def t_gameobject_template_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobject_template_locale"]

    @cached_property
    def t_gameobjectartkit_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobjectartkit_dbc"]

    @cached_property
    def t_gameobjectdisplayinfo_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gameobjectdisplayinfo_dbc"]

    @cached_property
    def t_gemproperties_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gemproperties_dbc"]

    @cached_property
    def t_glyphproperties_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.glyphproperties_dbc"]

    @cached_property
    def t_glyphslot_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.glyphslot_dbc"]

    @cached_property
    def t_gossip_menu(self) -> sa.Table:
        return self._metadata.tables["acore_world.gossip_menu"]

    @cached_property
    def t_gossip_menu_option(self) -> sa.Table:
        return self._metadata.tables["acore_world.gossip_menu_option"]

    @cached_property
    def t_gossip_menu_option_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.gossip_menu_option_locale"]

    @cached_property
    def t_graveyard_zone(self) -> sa.Table:
        return self._metadata.tables["acore_world.graveyard_zone"]

    @cached_property
    def t_gtbarbershopcostbase_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtbarbershopcostbase_dbc"]

    @cached_property
    def t_gtchancetomeleecrit_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtchancetomeleecrit_dbc"]

    @cached_property
    def t_gtchancetomeleecritbase_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtchancetomeleecritbase_dbc"]

    @cached_property
    def t_gtchancetospellcrit_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtchancetospellcrit_dbc"]

    @cached_property
    def t_gtchancetospellcritbase_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtchancetospellcritbase_dbc"]

    @cached_property
    def t_gtcombatratings_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtcombatratings_dbc"]

    @cached_property
    def t_gtnpcmanacostscaler_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtnpcmanacostscaler_dbc"]

    @cached_property
    def t_gtoctclasscombatratingscalar_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtoctclasscombatratingscalar_dbc"]

    @cached_property
    def t_gtoctregenhp_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtoctregenhp_dbc"]

    @cached_property
    def t_gtregenhpperspt_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtregenhpperspt_dbc"]

    @cached_property
    def t_gtregenmpperspt_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.gtregenmpperspt_dbc"]

    @cached_property
    def t_holiday_dates(self) -> sa.Table:
        return self._metadata.tables["acore_world.holiday_dates"]

    @cached_property
    def t_holidays_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.holidays_dbc"]

    @cached_property
    def t_instance_encounters(self) -> sa.Table:
        return self._metadata.tables["acore_world.instance_encounters"]

    @cached_property
    def t_instance_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.instance_template"]

    @cached_property
    def t_item_enchantment_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.item_enchantment_template"]

    @cached_property
    def t_item_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.item_loot_template"]

    @cached_property
    def t_item_set_names(self) -> sa.Table:
        return self._metadata.tables["acore_world.item_set_names"]

    @cached_property
    def t_item_set_names_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.item_set_names_locale"]

    @cached_property
    def t_item_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.item_template"]

    @cached_property
    def t_item_template_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.item_template_locale"]

    @cached_property
    def t_itembagfamily_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.itembagfamily_dbc"]

    @cached_property
    def t_itemdisplayinfo_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.itemdisplayinfo_dbc"]

    @cached_property
    def t_itemextendedcost_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.itemextendedcost_dbc"]

    @cached_property
    def t_itemlimitcategory_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.itemlimitcategory_dbc"]

    @cached_property
    def t_itemrandomproperties_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.itemrandomproperties_dbc"]

    @cached_property
    def t_itemrandomsuffix_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.itemrandomsuffix_dbc"]

    @cached_property
    def t_itemset_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.itemset_dbc"]

    @cached_property
    def t_lfg_dungeon_rewards(self) -> sa.Table:
        return self._metadata.tables["acore_world.lfg_dungeon_rewards"]

    @cached_property
    def t_lfg_dungeon_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.lfg_dungeon_template"]

    @cached_property
    def t_lfgdungeons_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.lfgdungeons_dbc"]

    @cached_property
    def t_light_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.light_dbc"]

    @cached_property
    def t_linked_respawn(self) -> sa.Table:
        return self._metadata.tables["acore_world.linked_respawn"]

    @cached_property
    def t_liquidtype_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.liquidtype_dbc"]

    @cached_property
    def t_lock_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.lock_dbc"]

    @cached_property
    def t_mail_level_reward(self) -> sa.Table:
        return self._metadata.tables["acore_world.mail_level_reward"]

    @cached_property
    def t_mail_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.mail_loot_template"]

    @cached_property
    def t_mailtemplate_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.mailtemplate_dbc"]

    @cached_property
    def t_map_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.map_dbc"]

    @cached_property
    def t_mapdifficulty_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.mapdifficulty_dbc"]

    @cached_property
    def t_milling_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.milling_loot_template"]

    @cached_property
    def t_movie_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.movie_dbc"]

    @cached_property
    def t_namesprofanity_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.namesprofanity_dbc"]

    @cached_property
    def t_namesreserved_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.namesreserved_dbc"]

    @cached_property
    def t_npc_spellclick_spells(self) -> sa.Table:
        return self._metadata.tables["acore_world.npc_spellclick_spells"]

    @cached_property
    def t_npc_text(self) -> sa.Table:
        return self._metadata.tables["acore_world.npc_text"]

    @cached_property
    def t_npc_text_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.npc_text_locale"]

    @cached_property
    def t_npc_trainer(self) -> sa.Table:
        return self._metadata.tables["acore_world.npc_trainer"]

    @cached_property
    def t_npc_vendor(self) -> sa.Table:
        return self._metadata.tables["acore_world.npc_vendor"]

    @cached_property
    def t_outdoorpvp_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.outdoorpvp_template"]

    @cached_property
    def t_overridespelldata_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.overridespelldata_dbc"]

    @cached_property
    def t_page_text(self) -> sa.Table:
        return self._metadata.tables["acore_world.page_text"]

    @cached_property
    def t_page_text_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.page_text_locale"]

    @cached_property
    def t_pet_levelstats(self) -> sa.Table:
        return self._metadata.tables["acore_world.pet_levelstats"]

    @cached_property
    def t_pet_name_generation(self) -> sa.Table:
        return self._metadata.tables["acore_world.pet_name_generation"]

    @cached_property
    def t_pet_name_generation_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.pet_name_generation_locale"]

    @cached_property
    def t_pickpocketing_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.pickpocketing_loot_template"]

    @cached_property
    def t_player_class_stats(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_class_stats"]

    @cached_property
    def t_player_classlevelstats(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_classlevelstats"]

    @cached_property
    def t_player_factionchange_achievement(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_factionchange_achievement"]

    @cached_property
    def t_player_factionchange_items(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_factionchange_items"]

    @cached_property
    def t_player_factionchange_quests(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_factionchange_quests"]

    @cached_property
    def t_player_factionchange_reputations(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_factionchange_reputations"]

    @cached_property
    def t_player_factionchange_spells(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_factionchange_spells"]

    @cached_property
    def t_player_factionchange_titles(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_factionchange_titles"]

    @cached_property
    def t_player_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_loot_template"]

    @cached_property
    def t_player_race_stats(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_race_stats"]

    @cached_property
    def t_player_xp_for_level(self) -> sa.Table:
        return self._metadata.tables["acore_world.player_xp_for_level"]

    @cached_property
    def t_playercreateinfo(self) -> sa.Table:
        return self._metadata.tables["acore_world.playercreateinfo"]

    @cached_property
    def t_playercreateinfo_action(self) -> sa.Table:
        return self._metadata.tables["acore_world.playercreateinfo_action"]

    @cached_property
    def t_playercreateinfo_cast_spell(self) -> sa.Table:
        return self._metadata.tables["acore_world.playercreateinfo_cast_spell"]

    @cached_property
    def t_playercreateinfo_item(self) -> sa.Table:
        return self._metadata.tables["acore_world.playercreateinfo_item"]

    @cached_property
    def t_playercreateinfo_skills(self) -> sa.Table:
        return self._metadata.tables["acore_world.playercreateinfo_skills"]

    @cached_property
    def t_playercreateinfo_spell_custom(self) -> sa.Table:
        return self._metadata.tables["acore_world.playercreateinfo_spell_custom"]

    @cached_property
    def t_points_of_interest(self) -> sa.Table:
        return self._metadata.tables["acore_world.points_of_interest"]

    @cached_property
    def t_points_of_interest_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.points_of_interest_locale"]

    @cached_property
    def t_pool_creature(self) -> sa.Table:
        return self._metadata.tables["acore_world.pool_creature"]

    @cached_property
    def t_pool_gameobject(self) -> sa.Table:
        return self._metadata.tables["acore_world.pool_gameobject"]

    @cached_property
    def t_pool_pool(self) -> sa.Table:
        return self._metadata.tables["acore_world.pool_pool"]

    @cached_property
    def t_pool_quest(self) -> sa.Table:
        return self._metadata.tables["acore_world.pool_quest"]

    @cached_property
    def t_pool_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.pool_template"]

    @cached_property
    def t_powerdisplay_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.powerdisplay_dbc"]

    @cached_property
    def t_prospecting_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.prospecting_loot_template"]

    @cached_property
    def t_pvpdifficulty_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.pvpdifficulty_dbc"]

    @cached_property
    def t_quest_details(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_details"]

    @cached_property
    def t_quest_greeting(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_greeting"]

    @cached_property
    def t_quest_greeting_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_greeting_locale"]

    @cached_property
    def t_quest_mail_sender(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_mail_sender"]

    @cached_property
    def t_quest_money_reward(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_money_reward"]

    @cached_property
    def t_quest_offer_reward(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_offer_reward"]

    @cached_property
    def t_quest_offer_reward_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_offer_reward_locale"]

    @cached_property
    def t_quest_poi(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_poi"]

    @cached_property
    def t_quest_poi_points(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_poi_points"]

    @cached_property
    def t_quest_request_items(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_request_items"]

    @cached_property
    def t_quest_request_items_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_request_items_locale"]

    @cached_property
    def t_quest_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_template"]

    @cached_property
    def t_quest_template_addon(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_template_addon"]

    @cached_property
    def t_quest_template_locale(self) -> sa.Table:
        return self._metadata.tables["acore_world.quest_template_locale"]

    @cached_property
    def t_questfactionreward_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.questfactionreward_dbc"]

    @cached_property
    def t_questsort_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.questsort_dbc"]

    @cached_property
    def t_questxp_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.questxp_dbc"]

    @cached_property
    def t_randproppoints_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.randproppoints_dbc"]

    @cached_property
    def t_reference_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.reference_loot_template"]

    @cached_property
    def t_reputation_reward_rate(self) -> sa.Table:
        return self._metadata.tables["acore_world.reputation_reward_rate"]

    @cached_property
    def t_reputation_spillover_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.reputation_spillover_template"]

    @cached_property
    def t_scalingstatdistribution_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.scalingstatdistribution_dbc"]

    @cached_property
    def t_scalingstatvalues_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.scalingstatvalues_dbc"]

    @cached_property
    def t_script_waypoint(self) -> sa.Table:
        return self._metadata.tables["acore_world.script_waypoint"]

    @cached_property
    def t_skill_discovery_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.skill_discovery_template"]

    @cached_property
    def t_skill_extra_item_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.skill_extra_item_template"]

    @cached_property
    def t_skill_fishing_base_level(self) -> sa.Table:
        return self._metadata.tables["acore_world.skill_fishing_base_level"]

    @cached_property
    def t_skill_perfect_item_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.skill_perfect_item_template"]

    @cached_property
    def t_skillline_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.skillline_dbc"]

    @cached_property
    def t_skilllineability_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.skilllineability_dbc"]

    @cached_property
    def t_skillraceclassinfo_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.skillraceclassinfo_dbc"]

    @cached_property
    def t_skilltiers_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.skilltiers_dbc"]

    @cached_property
    def t_skinning_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.skinning_loot_template"]

    @cached_property
    def t_smart_scripts(self) -> sa.Table:
        return self._metadata.tables["acore_world.smart_scripts"]

    @cached_property
    def t_soundentries_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.soundentries_dbc"]

    @cached_property
    def t_spell_area(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_area"]

    @cached_property
    def t_spell_bonus_data(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_bonus_data"]

    @cached_property
    def t_spell_cooldown_overrides(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_cooldown_overrides"]

    @cached_property
    def t_spell_custom_attr(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_custom_attr"]

    @cached_property
    def t_spell_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_dbc"]

    @cached_property
    def t_spell_enchant_proc_data(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_enchant_proc_data"]

    @cached_property
    def t_spell_group(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_group"]

    @cached_property
    def t_spell_group_stack_rules(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_group_stack_rules"]

    @cached_property
    def t_spell_linked_spell(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_linked_spell"]

    @cached_property
    def t_spell_loot_template(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_loot_template"]

    @cached_property
    def t_spell_mixology(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_mixology"]

    @cached_property
    def t_spell_pet_auras(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_pet_auras"]

    @cached_property
    def t_spell_proc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_proc"]

    @cached_property
    def t_spell_proc_event(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_proc_event"]

    @cached_property
    def t_spell_ranks(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_ranks"]

    @cached_property
    def t_spell_required(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_required"]

    @cached_property
    def t_spell_script_names(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_script_names"]

    @cached_property
    def t_spell_scripts(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_scripts"]

    @cached_property
    def t_spell_target_position(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_target_position"]

    @cached_property
    def t_spell_threat(self) -> sa.Table:
        return self._metadata.tables["acore_world.spell_threat"]

    @cached_property
    def t_spellcasttimes_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellcasttimes_dbc"]

    @cached_property
    def t_spellcategory_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellcategory_dbc"]

    @cached_property
    def t_spelldifficulty_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spelldifficulty_dbc"]

    @cached_property
    def t_spellduration_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellduration_dbc"]

    @cached_property
    def t_spellfocusobject_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellfocusobject_dbc"]

    @cached_property
    def t_spellitemenchantment_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellitemenchantment_dbc"]

    @cached_property
    def t_spellitemenchantmentcondition_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellitemenchantmentcondition_dbc"]

    @cached_property
    def t_spellradius_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellradius_dbc"]

    @cached_property
    def t_spellrange_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellrange_dbc"]

    @cached_property
    def t_spellrunecost_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellrunecost_dbc"]

    @cached_property
    def t_spellshapeshiftform_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellshapeshiftform_dbc"]

    @cached_property
    def t_spellvisual_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.spellvisual_dbc"]

    @cached_property
    def t_stableslotprices_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.stableslotprices_dbc"]

    @cached_property
    def t_summonproperties_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.summonproperties_dbc"]

    @cached_property
    def t_talent_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.talent_dbc"]

    @cached_property
    def t_talenttab_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.talenttab_dbc"]

    @cached_property
    def t_taxinodes_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.taxinodes_dbc"]

    @cached_property
    def t_taxipath_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.taxipath_dbc"]

    @cached_property
    def t_taxipathnode_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.taxipathnode_dbc"]

    @cached_property
    def t_teamcontributionpoints_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.teamcontributionpoints_dbc"]

    @cached_property
    def t_totemcategory_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.totemcategory_dbc"]

    @cached_property
    def t_transportanimation_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.transportanimation_dbc"]

    @cached_property
    def t_transportrotation_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.transportrotation_dbc"]

    @cached_property
    def t_transports(self) -> sa.Table:
        return self._metadata.tables["acore_world.transports"]

    @cached_property
    def t_updates(self) -> sa.Table:
        return self._metadata.tables["acore_world.updates"]

    @cached_property
    def t_updates_include(self) -> sa.Table:
        return self._metadata.tables["acore_world.updates_include"]

    @cached_property
    def t_vehicle_accessory(self) -> sa.Table:
        return self._metadata.tables["acore_world.vehicle_accessory"]

    @cached_property
    def t_vehicle_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.vehicle_dbc"]

    @cached_property
    def t_vehicle_template_accessory(self) -> sa.Table:
        return self._metadata.tables["acore_world.vehicle_template_accessory"]

    @cached_property
    def t_vehicleseat_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.vehicleseat_dbc"]

    @cached_property
    def t_version(self) -> sa.Table:
        return self._metadata.tables["acore_world.version"]

    @cached_property
    def t_warden_checks(self) -> sa.Table:
        return self._metadata.tables["acore_world.warden_checks"]

    @cached_property
    def t_waypoint_data(self) -> sa.Table:
        return self._metadata.tables["acore_world.waypoint_data"]

    @cached_property
    def t_waypoint_scripts(self) -> sa.Table:
        return self._metadata.tables["acore_world.waypoint_scripts"]

    @cached_property
    def t_waypoints(self) -> sa.Table:
        return self._metadata.tables["acore_world.waypoints"]

    @cached_property
    def t_wmoareatable_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.wmoareatable_dbc"]

    @cached_property
    def t_worldmaparea_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.worldmaparea_dbc"]

    @cached_property
    def t_worldmapoverlay_dbc(self) -> sa.Table:
        return self._metadata.tables["acore_world.worldmapoverlay_dbc"]



# -*- coding: utf-8 -*-

from .orm import Orm
from .orm_getter import get_orm_from_ec2_inside
from .orm_getter import get_orm_for_ssh_tunnel
from .orm_getter import get_orm_for_vpc
from .app import api as app
from .sdk import api as sdk



# -*- coding: utf-8 -*-

from acore_db_app import api


def test():
    _ = api

    _ = api.Orm
    _ = api.get_orm_from_ec2_inside
    _ = api.get_orm_for_ssh_tunnel
    _ = api.get_orm_for_vpc

    _ = api.app
    _ = api.app.LocaleEnum
    _ = api.app.quest.CharacterQuestStatusEnum
    _ = api.app.quest.CharacterQuestStatus
    _ = api.app.quest.list_quest_by_character
    _ = api.app.quest.EnrichedQuestData
    _ = api.app.quest.get_enriched_quest_data
    _ = api.app.quest.get_latest_n_quest_enriched_quest_data
    _ = api.app.quest.complete_latest_n_quest

    _ = api.sdk
    _ = api.sdk.quest.get_latest_n_request


if __name__ == "__main__":
    from acore_db_app.tests import run_cov_test

    run_cov_test(__file__, "acore_db_app.api", preview=False)



# -*- coding: utf-8 -*-

import typing as T

import json
import dataclasses

from boto_session_manager import BotoSesManager
from acore_server.api import Server

from ....compat import cached_property
from ....paths import path_gui_settings_json


@dataclasses.dataclass
class SettingObject:
    # static fields
    aws_profile: T.Optional[str] = dataclasses.field(default=None)
    server_id: str = dataclasses.field(default="sbx-blue")
    locale: str = dataclasses.field(default="enUS")

    # derived fields
    bsm: T.Optional[BotoSesManager] = dataclasses.field(default=None)
    server: T.Optional[Server] = dataclasses.field(default=None)

    def __post_init__(self):
        self.bsm = BotoSesManager(profile_name=self.aws_profile)
        if self.server_id:
            self.server = Server.get(
                bsm=self.bsm,
                server_id=self.server_id,
            )

    @classmethod
    def read_settings(cls):
        try:
            data = json.loads(path_gui_settings_json.read_text())
            return cls(**data)
        except FileNotFoundError:
            setting_object = cls()
            setting_object.write_settings()
            return setting_object

    def write_settings(self):
        data = {
            field: getattr(self, field)
            for field in self.static_fields
        }
        path_gui_settings_json.write_text(json.dumps(data, indent=4))

    @cached_property
    def static_fields(self) -> T.List[str]:
        return [
            "aws_profile",
            "server_id",
            "locale",
        ]

    def iter_static_items(self) -> T.Iterator[T.Tuple[str, str]]:
        for field in self.static_fields:
            yield field, getattr(self, field)


setting_object = SettingObject.read_settings()



# -*- coding: utf-8 -*-

from PySide6 import QtCore, QtWidgets, QtGui
from .query_form import QuestCompleterWidgetQueryForm
from .quest_selection import QuestCompleterWidgetQuestSelection
from .quest_action import QuestCompleterWidgetQuestAction


class QuestCompleterWidget(
    QtWidgets.QWidget,
    QuestCompleterWidgetQueryForm,
    QuestCompleterWidgetQuestSelection,
    QuestCompleterWidgetQuestAction,
):
    """
    Quest Completer 是主要帮助角色完成任务的 App.

    1. 一个顶部的搜索框, 用于搜索任务
    2. 一个中间左边的任务列表, 用于显示搜索结果
    3. 一个底部的动作列表, 用于点击后执行动作
    """

    def __init__(self, parent: QtWidgets.QWidget):
        super().__init__(parent)
        self.add_query_form()
        self.add_quest_selection()
        self.add_quest_action()
        self.set_layout()

    def set_layout(self):
        layout = QtWidgets.QVBoxLayout()
        layout.addLayout(self.query_form_layout)
        layout.addLayout(self.quest_selection_layout)
        layout.addLayout(self.quest_action_layout)
        self.setLayout(layout)



# -*- coding: utf-8 -*-

import typing as T
import json
from PySide6 import QtCore, QtWidgets, QtGui

from ....paths import path_gui_settings_json
from .setting_object import setting_object


class SettingsWidgetJsonSetting:
    def add_json_setting(self):
        self._add_json_setting_widgets()
        self._add_json_setting_layout()

    def _add_json_setting_widgets(self):
        # 创建一个用于展示 settings 中的 key value 的表格
        self.json_setting_form: T.Dict[
            str, T.Tuple[QtWidgets.QLabel, QtWidgets.QLineEdit]
        ] = {}

        # 从列表动态生成 key value 的表单
        for key, value in setting_object.iter_static_items():
            # 其中 key 是一个 label
            label = QtWidgets.QLabel(f"{key}:")
            label.setAlignment(QtCore.Qt.AlignCenter)
            label.setFixedWidth(120)

            # 而 value 是一个输入框
            edit = QtWidgets.QLineEdit()
            edit.setPlaceholderText(f"enter {key!r} here")
            # 默认第一次打开的时候会从 settings 中读取数据
            edit.setText(value)
            self.json_setting_form[key] = (label, edit)

        # 添加两个 button, 一个用于从 settings 中加载数据, 另一个用于将数据写入 settings
        self.load_button = QtWidgets.QPushButton("Load")
        self.load_button.clicked.connect(self._load_button_clicked_event_handler)
        self.apply_button = QtWidgets.QPushButton("Apply")
        self.apply_button.clicked.connect(self._apply_button_clicked_event_handler)

    @QtCore.Slot()
    def _load_button_clicked_event_handler(self):
        new_setting_object = setting_object.read_settings()
        for key, (label, edit) in self.json_setting_form.items():
            data = dict(new_setting_object.items())
            edit.setText(data[key])

    @QtCore.Slot()
    def _apply_button_clicked_event_handler(self):
        data = dict()
        for key, (label, edit) in self.json_setting_form.items():
            value = edit.text().strip()
            data[key] = value
            setattr(setting_object, key, value)
            print(f"apply settings {key!r} = {value!r}")
        print("write settings to json file")
        setting_object.write_settings()
        setting_object.__post_init__()
        # print(setting_object)

    def _add_json_setting_layout(self):
        # 将 json setting form 按照数列排版
        json_setting_form_layout = QtWidgets.QVBoxLayout()
        for key, (label, edit) in self.json_setting_form.items():
            json_setting_form_row_layout = QtWidgets.QHBoxLayout()
            json_setting_form_row_layout.addWidget(label)
            json_setting_form_row_layout.addWidget(edit)
            json_setting_form_layout.addLayout(json_setting_form_row_layout)

        # button 的 layout 也是一个子 layout
        button_layout = QtWidgets.QHBoxLayout()
        button_layout.addWidget(self.load_button)
        button_layout.addWidget(self.apply_button)

        # 将 json setting form 和 button 组合起来
        self.json_setting_layout = QtWidgets.QVBoxLayout()
        self.json_setting_layout.addLayout(json_setting_form_layout)
        self.json_setting_layout.addLayout(button_layout)



# -*- coding: utf-8 -*-

import typing as T

from PySide6 import QtCore, QtWidgets, QtGui

from ....sdk.api import quest
from ..settings.json_setting import setting_object


if T.TYPE_CHECKING:
    from .main import QuestCompleterWidget


from boto_session_manager import BotoSesManager

bsm = BotoSesManager(profile_name="bmt_app_dev_us_east_1")


class QuestCompleterWidgetQueryForm:
    def add_query_form(self):
        self._add_query_form_widgets()
        self._add_query_form_layout()

    def _add_query_form_widgets(self):
        self.query_form_header = QtWidgets.QLabel("Search Quest")
        self.query_form_header.setAlignment(QtCore.Qt.AlignCenter)

        self.query_form_character_label = QtWidgets.QLabel("Character")
        self.query_form_character_label.setAlignment(QtCore.Qt.AlignCenter)
        self.query_form_character_label.setFixedWidth(60)
        self.query_form_character_value = QtWidgets.QLineEdit()
        self.query_form_character_value.setPlaceholderText("enter Character name here")

        self.search_button = QtWidgets.QPushButton("Search")
        self.search_button.clicked.connect(self._search_button_clicked_event_handler)

    def _search_button_clicked_event_handler(self: "QuestCompleterWidget"):
        print("search button clicked")
        instance_id = setting_object.server.metadata.ec2_inst.id
        character = self.query_form_character_value.text()
        locale = setting_object.locale
        print(f"search quests for: character = {character!r}, locale = {locale!r} on server_id = {setting_object.server.id!r}")
        enriched_quest_data_list = quest.get_latest_n_request(
            bsm=setting_object.bsm,
            instance_id=instance_id,
            character=character,
            locale=locale,
            n=25,
        )

        self.quest_selection_item_list.clear()
        for enriched_quest_data in enriched_quest_data_list:
            item = QtWidgets.QListWidgetItem(f"{enriched_quest_data.quest_id} {enriched_quest_data.quest_title}")
            item.enriched_quest_data = enriched_quest_data
            item.setTextAlignment(QtCore.Qt.AlignLeft)
            self.quest_selection_item_list.addItem(item)

    def _add_query_form_layout(self):
        self.query_form_layout = QtWidgets.QVBoxLayout()

        self.query_form_layout.addWidget(self.query_form_header)

        pairs = [
            (self.query_form_character_label, self.query_form_character_value),
        ]
        for label, value in pairs:
            query_form_row_layout = QtWidgets.QHBoxLayout()
            query_form_row_layout.addWidget(label)
            query_form_row_layout.addWidget(value)
            self.query_form_layout.addLayout(query_form_row_layout)

        self.query_form_layout.addWidget(self.search_button)



# -*- coding: utf-8 -*-


import json
import dataclasses

from ..app import api as app
from ..orm_getter import get_orm_from_ec2_inside


def get_latest_n_quest(
    character: str,
    locale: str = app.LocaleEnum.enUS.value,
    n: int = 3,
):
    filtered_enriched_quest_data_list = app.quest.get_latest_n_quest_enriched_quest_data(
        orm=get_orm_from_ec2_inside(),
        character=character,
        locale=app.LocaleEnum[locale],
        n=n,
    )
    print(
        json.dumps(
            [
                dataclasses.asdict(enriched_quest_data)
                for enriched_quest_data in filtered_enriched_quest_data_list
            ],
            ensure_ascii=False,
        )
    )



# -*- coding: utf-8 -*-

# 任何面向最终用户的 app 基本上都是从命令行接入的, 所以 sys 模块是必须的
import sys

# QtCore, QtWidgets, QtGui 是 PySide 的三个主要模块, 一般每一个项目都会用到这三个模块.
# 所以我一般在开始就会 Import 它们, 不管用不用得到.
# 全部的模块列表: https://doc.qt.io/qtforpython-6/modules.html#
from PySide6 import QtCore, QtWidgets, QtGui

from .tabs.main import TabDialog


class MainWindow(QtWidgets.QMainWindow):
    """
    一般任何一个 App 都有一个主窗口. 在这个主窗口里我们可以塞下各种各样的 Widget.
    """

    def __init__(self, widget: QtWidgets.QWidget):
        QtWidgets.QMainWindow.__init__(self)
        self.setWindowTitle("Acore Database App")
        self.add_menu()
        self.setCentralWidget(widget)

    def add_menu(self):
        # Create the tray
        self.tray = QtWidgets.QSystemTrayIcon()
        self.tray.setIcon(QtGui.QIcon("icon.png"))
        self.tray.setVisible(True)

        # Create the menu
        self.menu = QtWidgets.QMenu()
        self.file_menu = self.menu.addMenu("File")

        settings_action = QtGui.QAction("settings", self)
        settings_action.triggered.connect(self.update_settings)

        self.file_menu.addAction(settings_action)
        self.tray.setContextMenu(self.menu)

    @QtCore.Slot()
    def update_settings(self, checked):
        print("update settings")


def run():
    app = QtWidgets.QApplication(sys.argv)

    tab_dialog = TabDialog()

    window = MainWindow(tab_dialog)  # 将主要的 widget 添加到 MainWindow 中
    window.resize(1200, 800)
    window.show()

    sys.exit(app.exec())



# -*- coding: utf-8 -*-

import typing as T

from PySide6 import QtCore, QtWidgets, QtGui

from ....sdk.api import quest

if T.TYPE_CHECKING:
    from .main import QuestCompleterWidget


class QuestCompleterWidgetQuestSelection:
    def add_quest_selection(self):
        self._add_quest_selection_widgets()
        self._add_quest_selection_layout()

    def _add_quest_selection_widgets(self):
        self.quest_selection_item_list_header = QtWidgets.QLabel("Quests")
        self.quest_selection_item_list_header.setAlignment(QtCore.Qt.AlignCenter)

        self.quest_selection_item_list = QtWidgets.QListWidget()
        self.quest_selection_item_list.setSelectionMode(
            QtWidgets.QAbstractItemView.SelectionMode.SingleSelection
        )

        self.quest_selection_item_list.itemSelectionChanged.connect(
            self._quest_selection_item_selection_changed_event_handler
        )

    def _quest_selection_item_selection_changed_event_handler(self: "QuestCompleterWidget"):
        """
        当用户双击 items 中的某个 item 的时候会触发这个函数.
        """
        selected_items = self.quest_selection_item_list.selectedItems()
        item = selected_items[0]
        text = item.text()
        print(f"item list item selection changed: {text!r}")

        self.quest_action_item_list.clear()

        enriched_quest_data: quest.EnrichedQuestData = item.enriched_quest_data

        copy_text = f".quest complete {enriched_quest_data.quest_id}"
        item = QtWidgets.QListWidgetItem(f"double click to copy COMPLETE QUEST command: {copy_text}")
        item.copy_text = copy_text
        item.setTextAlignment(QtCore.Qt.AlignLeft)
        self.quest_action_item_list.addItem(item)

        copy_text = (
            f".go xyz "
            f"{enriched_quest_data.starter_position_x} "
            f"{enriched_quest_data.starter_position_y} "
            f"{enriched_quest_data.starter_position_z} "
            f"{enriched_quest_data.starter_map}"
        )
        item = QtWidgets.QListWidgetItem(f"double click to copy TELEPORT TO QUEST STARTER command: {copy_text}")
        item.copy_text = copy_text
        item.setTextAlignment(QtCore.Qt.AlignLeft)
        self.quest_action_item_list.addItem(item)

        copy_text = (
            f".go xyz "
            f"{enriched_quest_data.ender_position_x} "
            f"{enriched_quest_data.ender_position_y} "
            f"{enriched_quest_data.ender_position_z} "
            f"{enriched_quest_data.ender_map}"
        )
        item = QtWidgets.QListWidgetItem(f"double click to copy TELEPORT TO QUEST ENDER command: {copy_text}")
        item.copy_text = copy_text
        item.setTextAlignment(QtCore.Qt.AlignLeft)
        self.quest_action_item_list.addItem(item)

    def _add_quest_selection_layout(self):
        self.quest_selection_layout = QtWidgets.QVBoxLayout()
        self.quest_selection_layout.addWidget(self.quest_selection_item_list_header)
        self.quest_selection_layout.addWidget(self.quest_selection_item_list)



# -*- coding: utf-8 -*-

"""
该模块用于实现 DB App 的具体业务逻辑.

**开发者说明**

本模块下的所有具体业务逻辑都会被封装成函数. 每个函数必定带有一个 :class:`~acore_db_app.orm.Orm`
的输入参数. 这个参数绑定了数据库连接以及所有的数据表定义. 换言之, 我们的底层函数实现并不关心具体
跟哪个数据库建立连接. 因为底层函数是被作为类库来设计的, 最终的用户自己需要决定跟哪个数据库建立连接.
"""



# -*- coding: utf-8 -*-

"""
该模块用于创建 ORM 对象的实例. 该模块的函数会根据当前运行环境的不同, 选择不同的方式来创建 ORM
对象的实例.
"""

from boto_session_manager import BotoSesManager
from acore_db_ssh_tunnel.api import create_engine
from acore_server.api import Server

from .orm import Orm
from .cache import cache


DB_INFO_CACHE_EXPIRE = 3600


@cache.memoize(name="db_info_from_ec2_inside", expire=DB_INFO_CACHE_EXPIRE)
def _get_db_info_from_ec2_inside() -> dict:
    server = Server.from_ec2_inside()
    return {
        "db_host": server.metadata.rds_inst.endpoint,
        "db_username": server.config.db_username,
        "db_password": server.config.db_password,
    }


def get_orm_from_ec2_inside() -> Orm:
    """
    从 EC2 实例内部获取数据库信息, 并创建 ORM 对象的实例.
    """
    db_info = _get_db_info_from_ec2_inside()
    engine = create_engine(
        host=db_info["db_host"],
        port=3306,
        username=db_info["db_username"],
        password=db_info["db_password"],
        db_name="acore_auth",
    )
    return Orm(engine=engine)


def _get_db_info_from_ec2_outside(
    bsm: BotoSesManager,
    server_id: str,
) -> dict:
    key = "db_info_from_ec2_outside"
    value = cache.get(key)
    if value is None:
        server = Server.get(bsm=bsm, server_id=server_id)
        value = {
            "db_host": server.metadata.rds_inst.endpoint,
            "db_username": server.config.db_username,
            "db_password": server.config.db_password,
        }
        cache.set(key=key, value=value, expire=DB_INFO_CACHE_EXPIRE)
    return value


def get_orm_for_ssh_tunnel(
    bsm: BotoSesManager,
    server_id: str,
) -> Orm:
    """
    创建基于 SSH Tunnel 的 ORM 对象的实例. 该函数常用于在本地开发电脑上连接数据库.

    :param bsm: BotoSesManager 对象的实例.
    :param server_id: 服务器 ID. Example: ``${env_name}-${server_name}``.
    """
    db_info = _get_db_info_from_ec2_outside(bsm=bsm, server_id=server_id)
    engine = create_engine(
        host="127.0.0.1",
        port=3306,
        username=db_info["db_username"],
        password=db_info["db_password"],
        db_name="acore_auth",
    )
    return Orm(engine=engine)


def get_orm_for_vpc(
    bsm: BotoSesManager,
    server_id: str,
) -> Orm:
    """
    创建基于 VPC 的 ORM 对象的实例. 该函数常用于在与数据库同处于一个 VPC 下的 EC2 或 Lambda
    中连接数据库.

    :param bsm: BotoSesManager 对象的实例.
    :param server_id: 服务器 ID. Example: ``${env_name}-${server_name}``.
    """
    db_info = _get_db_info_from_ec2_outside(bsm=bsm, server_id=server_id)
    engine = create_engine(
        host=db_info["db_host"],
        port=3306,
        username=db_info["db_username"],
        password=db_info["db_password"],
        db_name="acore_auth",
    )
    return Orm(engine=engine)



# -*- coding: utf-8 -*-

"""
Acore DB App CLI interface
"""

import fire

from ..app.api import LocaleEnum


class Quest:
    """
    A collection of canned SOAP Agent commands.
    """

    def get_latest_n_quest(
        self,
        char: str,
        locale: str = LocaleEnum.enUS.value,
        n: int = 3,
    ):
        """
        Get the online players and characters in world. Also, you can use this
         command to check whether server is online.

        Example::

            acoredb quest get_latest_n_quest --help

            acoredb quest get_latest_n_quest --char mychar --locale enUS --n 3
        """
        # 注: 这段代码不能放在文件开头, 因为这段代码会 import cache. 如果我们放在文件开头,
        # 那么在 bootstrap 的时候是 root user 创建的 cache 数据库, 会导致普通用户没有权限
        # 使用
        from .impl import get_latest_n_quest

        get_latest_n_quest(
            character=char,
            locale=locale,
            n=n,
        )


class Command:
    """
    Example:

    - acoredb
    """

    def __init__(self):
        self.quest = Quest()

    def hello(self):
        """
        Print welcome message.
        """
        print("Hello acore db app user!")


def run():
    fire.Fire(Command)



# -*- coding: utf-8 -*-

from PySide6 import QtCore, QtWidgets, QtGui

from .quest_completer.main import QuestCompleterWidget
from .settings.main import SettingsWidget


class TabDialog(QtWidgets.QDialog):
    """
    这是我们的 App 的主要的 Widget, 它包含了几个 Tab.

    1. Quest Completer Tab, 用于搜索任务, 并且完成任务
    2. Settings Tab, 主要 GUI 的配置管理

    每个 Tab 本身是一个子 Widget.
    """

    def __init__(self, parent: QtWidgets.QWidget = None):
        super().__init__(parent)
        self.tab_widget = QtWidgets.QTabWidget()

        # 一共有两个 Tab, 一个用于搜索, 一个用于设置
        self.tab_widget.addTab(QuestCompleterWidget(self), "Quest Completer")
        self.tab_widget.addTab(SettingsWidget(self), "Settings")

        # 这个 Widget 只有一个 Layout, 用于放置 Tab
        main_layout = QtWidgets.QVBoxLayout()
        main_layout.addWidget(self.tab_widget)
        self.setLayout(main_layout)



# -*- coding: utf-8 -*-

from PySide6 import QtCore, QtWidgets, QtGui
from .json_setting import SettingsWidgetJsonSetting

# from .quest_selection import QuestCompleterWidgetQuestSelection
# from .quest_action import QuestCompleterWidgetQuestAction


class SettingsWidget(
    QtWidgets.QWidget,
    SettingsWidgetJsonSetting,
):
    """
    Settings 主要是用来修改我们的 GUI.
    """

    def __init__(self, parent: QtWidgets.QWidget):
        super().__init__(parent)
        self.add_json_setting()
        self.set_layout()

    def set_layout(self):
        layout = QtWidgets.QVBoxLayout()
        layout.addLayout(self.json_setting_layout)
        self.setLayout(layout)



# -*- coding: utf-8 -*-

"""
该模块用于实现跟 Quest 相关的 App
"""

import typing as T
import enum
import dataclasses

import sqlalchemy as sa

from ..orm import Orm
from ..logger import logger

from .locale import LocaleEnum


def _normalize_character(name: str) -> str:
    """
    将角色名称标准化为数据库中的值. 第一个字母大写, 后面的字母小写.
    """
    return name[0].upper() + name[1:].lower()


class CharacterQuestStatusEnum(int, enum.Enum):
    QUEST_STATUS_NONE = 0  # Quest isn't shown in quest list; default
    QUEST_STATUS_COMPLETE = 1  # Quest has been completed
    QUEST_STATUS_UNAVAILABLE = 2  # NOT USED
    QUEST_STATUS_INCOMPLETE = 3  # Quest is active in quest log but incomplete
    QUEST_STATUS_AVAILABLE = 4  # NOT USED
    QUEST_STATUS_FAILED = 5  # Player failed to complete the quest


@dataclasses.dataclass
class CharacterQuestStatus:
    """
    Reference: https://www.azerothcore.org/wiki/character_queststatus

    :param quest: Quest ID
    :param status: Quest status code, see :class:`CharacterQuestStatusEnum`
    """

    quest: int
    status: int

    def is_complete(self) -> bool:
        return self.status == CharacterQuestStatusEnum.QUEST_STATUS_COMPLETE.value

    def is_incomplete(self) -> bool:
        return self.status == CharacterQuestStatusEnum.QUEST_STATUS_INCOMPLETE.value

    def is_failed(self) -> bool:
        return self.status == CharacterQuestStatusEnum.QUEST_STATUS_FAILED.value


def list_quest_by_character(
    orm: Orm,
    character: str,
) -> T.List[CharacterQuestStatus]:
    """
    列出指定角色的所有任务的状态信息. 按照接任务的事件排序, 最新的任务在最前面.
    """
    with orm.engine.connect() as connect:
        stmt = (
            sa.select(
                orm.t_character_queststatus.c.quest,
                orm.t_character_queststatus.c.status,
            )
            .select_from(
                orm.t_characters.join(
                    orm.t_character_queststatus,
                    orm.t_character_queststatus.c.guid == orm.t_characters.c.guid,
                )
            )
            .where(orm.t_characters.c.name == _normalize_character(character))
            .order_by(orm.t_character_queststatus.c.timer.desc())
        )
        return [CharacterQuestStatus(**row) for row in connect.execute(stmt).mappings()]


@dataclasses.dataclass
class EnrichedQuestData:
    """
    代表着一个魔兽世界任务的数据. 并将额外的相关信息也整合到了一起. 该 data model 不包含与
    玩家相关的信息, 例如任务状态, 任务是否完成等等. 它是一个完全无状态的数据.
    """

    quest_id: T.Optional[int] = dataclasses.field(default=None)
    quest_title_enUS: T.Optional[str] = dataclasses.field(default=None)
    quest_title_locale: T.Optional[str] = dataclasses.field(default=None)
    locale: T.Optional[str] = dataclasses.field(default=None)
    starter_creature_id: T.Optional[int] = dataclasses.field(default=None)
    starter_guid: T.Optional[int] = dataclasses.field(default=None)
    starter_position_x: T.Optional[float] = dataclasses.field(default=None)
    starter_position_y: T.Optional[float] = dataclasses.field(default=None)
    starter_position_z: T.Optional[float] = dataclasses.field(default=None)
    starter_map: T.Optional[int] = dataclasses.field(default=None)
    ender_creature_id: T.Optional[int] = dataclasses.field(default=None)
    ender_guid: T.Optional[int] = dataclasses.field(default=None)
    ender_position_x: T.Optional[float] = dataclasses.field(default=None)
    ender_position_y: T.Optional[float] = dataclasses.field(default=None)
    ender_position_z: T.Optional[float] = dataclasses.field(default=None)
    ender_map: T.Optional[int] = dataclasses.field(default=None)

    @property
    def quest_title(self) -> str:
        if self.quest_title_locale is None:
            return self.quest_title_enUS
        else:
            return self.quest_title_locale

    @logger.pretty_log()
    def get_gm_commands(self):
        quest_complete_cmd = f".quest complete {self.quest_id}"
        go_starter_cmd = ".go xyz {} {} {} {}".format(
            self.starter_position_x,
            self.starter_position_y,
            self.starter_position_z,
            self.starter_map,
        )
        go_ender_cmd = ".go xyz {} {} {} {}".format(
            self.ender_position_x,
            self.ender_position_y,
            self.ender_position_z,
            self.ender_map,
        )
        logger.info(f"完成任务 {self.quest_title!r}:")
        logger.info(f"  {quest_complete_cmd}")
        logger.info("传送至任务开始 NPC:")
        logger.info(f"  {go_starter_cmd}")
        logger.info("传送至任务结束 NPC:")
        logger.info(f"  {go_ender_cmd}")


def get_enriched_quest_data(
    orm: Orm,
    character: str,
    locale: LocaleEnum = LocaleEnum.enUS,
    quest_title: T.Optional[str] = None,
    quest_objective: T.Optional[str] = None,
    quest_detail: T.Optional[str] = None,
    limit: int = 25,
) -> T.List[EnrichedQuestData]:
    """
    给定数据库连接, 和一个魔兽世界游戏角色的名字. 根据这些信息对获得该玩家所有的任务的详细信息,
    并将跟任务相关的其他数据都整合到一起.

    :param orm:
    :param character: 魔兽世界角色名字
    :param locale: 本地化语言, 默认为英文
    :param quest_title: 可选参数, 根据任务标题对任务进行过滤
    :param quest_objective: 可选参数, 根据任务目标对任务进行过滤
    :param quest_detail: 可选参数, 根据任务详情对任务进行过滤
    :param limit: 返回的任务数量的最大限制
    """
    with orm.engine.connect() as connect:
        selects = list()
        selects.append(orm.t_character_queststatus.c.quest.label("quest_id"))

        wheres = list()
        wheres.append(orm.t_characters.c.name == _normalize_character(character))

        joins = orm.t_characters.join(
            # 获得只属于该角色的任务状态
            orm.t_character_queststatus,
            orm.t_character_queststatus.c.guid == orm.t_characters.c.guid,
        )

        # 获得任务的英文文本信息
        selects.append(orm.t_quest_template.c.LogTitle.label("quest_title_enUS"))
        joins = joins.join(
            orm.t_quest_template,
            orm.t_quest_template.c.ID == orm.t_character_queststatus.c.quest,
        )

        # 若指定了除英文以外的语言, 则获得任务的本地化文本信息
        # the main table has to be on the left
        if locale is not LocaleEnum.enUS:
            # 先对 quest_template_locale 子表进行 filter, 只要指定的 locale
            subquery = sa.select(
                orm.t_quest_template_locale.c.ID,
                orm.t_quest_template_locale.c.Title,
            ).where(
                orm.t_quest_template_locale.c.locale == locale.value,
            ).subquery()

            selects.extend([
                subquery.c.Title.label("quest_title_locale"),
            ])

            # 然后再进行 left outer JOIN
            joins = joins.join(
                # 获得任务的文本信息
                subquery,
                subquery.c.ID == orm.t_character_queststatus.c.quest,
                isouter=True,
            )

        t_creature_quest_starter_entry = orm.t_creature.alias()
        t_creature_quest_ender_entry = orm.t_creature.alias()

        selects.extend(
            [
                orm.t_creature_queststarter.c.id.label("starter_creature_id"),
                t_creature_quest_starter_entry.c.guid.label("starter_guid"),
                t_creature_quest_starter_entry.c.position_x.label("starter_position_x"),
                t_creature_quest_starter_entry.c.position_y.label("starter_position_y"),
                t_creature_quest_starter_entry.c.position_z.label("starter_position_z"),
                t_creature_quest_starter_entry.c.map.label("starter_map"),
                orm.t_creature_questender.c.id.label("ender_creature_id"),
                t_creature_quest_ender_entry.c.guid.label("ender_guid"),
                t_creature_quest_ender_entry.c.position_x.label("ender_position_x"),
                t_creature_quest_ender_entry.c.position_y.label("ender_position_y"),
                t_creature_quest_ender_entry.c.position_z.label("ender_position_z"),
                t_creature_quest_ender_entry.c.map.label("ender_map"),
            ]
        )

        joins = (
            joins.join(
                # 获得任务给与者的信息
                orm.t_creature_queststarter,
                orm.t_creature_queststarter.c.quest
                == orm.t_character_queststatus.c.quest,
            )
            .join(
                # 获得任务结束者的信息
                orm.t_creature_questender,
                orm.t_creature_questender.c.quest
                == orm.t_character_queststatus.c.quest,
            )
            .join(
                # 获得任务给与者 NPC 的坐标信息
                t_creature_quest_starter_entry,
                t_creature_quest_starter_entry.c.id1
                == orm.t_creature_queststarter.c.id,
            )
            .join(
                # 获得任务结束者 NPC 的坐标信息
                t_creature_quest_ender_entry,
                t_creature_quest_ender_entry.c.id1 == orm.t_creature_queststarter.c.id,
            )
        )


        if quest_title is not None:
            if locale is LocaleEnum.enUS:
                wheres.append(orm.t_quest_template.c.Title.like(f"%{quest_title}%"))
            else:
                wheres.append(
                    orm.t_quest_template_locale.c.Title.like(f"%{quest_title}%")
                )
        if quest_objective is not None:
            if locale is LocaleEnum.enUS:
                wheres.append(orm.t_quest_template.c.Title.like(f"%{quest_title}%"))
            else:
                wheres.append(
                    orm.t_quest_template_locale.c.Objectives.like(
                        f"%{quest_objective}%"
                    )
                )
        if quest_detail is not None:
            if locale is LocaleEnum.enUS:
                wheres.append(orm.t_quest_template.c.Title.like(f"%{quest_title}%"))
            else:
                wheres.append(
                    orm.t_quest_template_locale.c.Details.like(f"%{quest_detail}%")
                )

        stmt = sa.select(*selects).select_from(joins).where(*wheres).limit(limit)
        # print(stmt)
        enriched_quest_data_list = list()
        for row in connect.execute(stmt).mappings():
            # print(row)
            enriched_quest_data = EnrichedQuestData(**row)
            enriched_quest_data_list.append(enriched_quest_data)
        return enriched_quest_data_list


def get_latest_n_quest_enriched_quest_data(
    orm: Orm,
    character: str,
    locale: LocaleEnum,
    n: int = 3,
) -> T.List[EnrichedQuestData]:
    """
    给定数据库连接, 和一个魔兽世界游戏角色的名字. 根据这些信息对获得该玩家所有的任务的详细信息,
    并将跟任务相关的其他数据都整合到一起. 最终按照接任务的事件顺序返回最新的 n 个任务的详细信息.
    """
    character_quest_status_list = list_quest_by_character(
        orm=orm,
        character=character,
    )
    enriched_quest_data_list = get_enriched_quest_data(
        orm=orm,
        character=character,
        locale=locale,
    )
    enriched_quest_data_mapper = {
        enriched_quest_data.quest_id: enriched_quest_data
        for enriched_quest_data in enriched_quest_data_list
    }
    ordered_enriched_quest_data_list = [
        enriched_quest_data_mapper[character_quest_status.quest]
        for character_quest_status in character_quest_status_list
        if character_quest_status.quest in enriched_quest_data_mapper
    ]
    ordered_enriched_quest_data_list = ordered_enriched_quest_data_list[:n]
    return ordered_enriched_quest_data_list


@logger.pretty_log()
def _print_complete_latest_n_quest_gm_commands(
    character: str,
    n: int,
    enriched_quest_data_list: T.List[EnrichedQuestData],
):
    logger.info(f"打印完成 {character!r} 的最新的 {n} 个任务的 GM 命令:")
    for enriched_quest_data in enriched_quest_data_list:
        with logger.nested():
            enriched_quest_data.get_gm_commands()


def complete_latest_n_quest(
    orm: Orm,
    character: str,
    locale: LocaleEnum,
    n: int = 3,
):
    """
    给定数据库连接, 和一个魔兽世界游戏角色的名字. 打印完成该玩家最新的 n 个任务的 GM 命令.
    其中包含完成任务的命令, 传送到接任务, 交任务的 NPC 处的命令.
    """
    ordered_enriched_quest_data_list = get_latest_n_quest_enriched_quest_data(
        orm=orm,
        character=character,
        locale=locale,
        n=n,
    )
    _print_complete_latest_n_quest_gm_commands(
        character=character,
        n=n,
        enriched_quest_data_list=ordered_enriched_quest_data_list,
    )



# -*- coding: utf-8 -*-

import typing as T
import json

from boto_session_manager import BotoSesManager
from acore_paths.api import path_acore_db_app_cli
import aws_ssm_run_command.api as aws_ssm_run_command

from ..app.api import quest


def get_latest_n_request(
    bsm: BotoSesManager,
    instance_id: str,
    character: str,
    locale: str,
    n: int,
) -> T.List[quest.EnrichedQuestData]:
    command_invocation = aws_ssm_run_command.better_boto.send_command_sync(
        ssm_client=bsm.ssm_client,
        instance_id=instance_id,
        commands=[
            (
                f"{path_acore_db_app_cli} "
                f"quest get-latest-n-quest --char {character} --locale {locale} --n {n}"
            ),
        ],
        delays=1,
        timeout=10,
    )
    enriched_quest_data_list = [
        quest.EnrichedQuestData(**dct)
        for dct in json.loads(command_invocation.StandardOutputContent)
    ]
    return enriched_quest_data_list



# -*- coding: utf-8 -*-

"""
在 ``orm.py`` 模块中, 我们需要对所有的 table 进行枚举. 这么多 Table 用手写也太麻烦了,
所以我做了这个模块用于动态生成所需的代码. 目前看来不同的数据库中没有名字一样的表, 所以我们可以
就用 table 的名字来作为枚举的名字.
"""

from pathlib_mate import Path

auth_tables = """
account
account_access
account_banned
account_muted
autobroadcast
build_info
ip_banned
logs
logs_ip_actions
motd
realmcharacters
realmlist
secret_digest
updates
updates_include
uptime
"""

char_tables = """
account_data
account_instance_times
account_tutorial
addons
arena_team
arena_team_member
auctionhouse
banned_addons
battleground_deserters
bugreport
calendar_events
calendar_invites
channels
channels_bans
channels_rights
character_account_data
character_achievement
character_achievement_progress
character_action
character_arena_stats
character_aura
character_banned
character_battleground_random
character_brew_of_the_month
character_declinedname
character_entry_point
character_equipmentsets
character_gifts
character_glyphs
character_homebind
character_instance
character_inventory
character_pet
character_pet_declinedname
character_queststatus
character_queststatus_daily
character_queststatus_monthly
character_queststatus_rewarded
character_queststatus_seasonal
character_queststatus_weekly
character_reputation
character_settings
character_skills
character_social
character_spell
character_spell_cooldown
character_stats
character_talent
characters
corpse
creature_respawn
game_event_condition_save
game_event_save
gameobject_respawn
gm_subsurvey
gm_survey
gm_ticket
group_member
groups
guild
guild_bank_eventlog
guild_bank_item
guild_bank_right
guild_bank_tab
guild_eventlog
guild_member
guild_member_withdraw
guild_rank
instance
instance_reset
instance_saved_go_state_data
item_instance
item_loot_storage
item_refund_instance
item_soulbound_trade_data
lag_reports
lfg_data
log_arena_fights
log_arena_memberstats
log_encounter
log_money
mail
mail_items
mail_server_character
mail_server_template
pet_aura
pet_spell
pet_spell_cooldown
petition
petition_sign
pool_quest_save
profanity_name
pvpstats_battlegrounds
pvpstats_players
quest_tracker
recovery_item
reserved_name
updates
updates_include
warden_action
worldstates
"""

world_tables = """
achievement_category_dbc
achievement_criteria_data
achievement_criteria_dbc
achievement_dbc
achievement_reward
achievement_reward_locale
acore_string
areagroup_dbc
areapoi_dbc
areatable_dbc
areatrigger
areatrigger_involvedrelation
areatrigger_scripts
areatrigger_tavern
areatrigger_teleport
auctionhouse_dbc
bankbagslotprices_dbc
barbershopstyle_dbc
battleground_template
battlemaster_entry
battlemasterlist_dbc
broadcast_text
broadcast_text_locale
charstartoutfit_dbc
chartitles_dbc
chatchannels_dbc
chrclasses_dbc
chrraces_dbc
cinematiccamera_dbc
cinematicsequences_dbc
command
conditions
creature
creature_addon
creature_classlevelstats
creature_equip_template
creature_formations
creature_loot_template
creature_model_info
creature_movement_override
creature_onkill_reputation
creature_questender
creature_questitem
creature_queststarter
creature_summon_groups
creature_template
creature_template_addon
creature_template_locale
creature_template_movement
creature_template_resistance
creature_template_spell
creature_text
creature_text_locale
creaturedisplayinfo_dbc
creaturedisplayinfoextra_dbc
creaturefamily_dbc
creaturemodeldata_dbc
creaturespelldata_dbc
creaturetype_dbc
currencytypes_dbc
destructiblemodeldata_dbc
disables
disenchant_loot_template
dungeon_access_requirements
dungeon_access_template
dungeonencounter_dbc
durabilitycosts_dbc
durabilityquality_dbc
emotes_dbc
emotestext_dbc
event_scripts
exploration_basexp
faction_dbc
factiontemplate_dbc
fishing_loot_template
game_event
game_event_arena_seasons
game_event_battleground_holiday
game_event_condition
game_event_creature
game_event_creature_quest
game_event_gameobject
game_event_gameobject_quest
game_event_model_equip
game_event_npc_vendor
game_event_npcflag
game_event_pool
game_event_prerequisite
game_event_quest_condition
game_event_seasonal_questrelation
game_graveyard
game_tele
game_weather
gameobject
gameobject_addon
gameobject_loot_template
gameobject_questender
gameobject_questitem
gameobject_queststarter
gameobject_template
gameobject_template_addon
gameobject_template_locale
gameobjectartkit_dbc
gameobjectdisplayinfo_dbc
gemproperties_dbc
glyphproperties_dbc
glyphslot_dbc
gossip_menu
gossip_menu_option
gossip_menu_option_locale
graveyard_zone
gtbarbershopcostbase_dbc
gtchancetomeleecrit_dbc
gtchancetomeleecritbase_dbc
gtchancetospellcrit_dbc
gtchancetospellcritbase_dbc
gtcombatratings_dbc
gtnpcmanacostscaler_dbc
gtoctclasscombatratingscalar_dbc
gtoctregenhp_dbc
gtregenhpperspt_dbc
gtregenmpperspt_dbc
holiday_dates
holidays_dbc
instance_encounters
instance_template
item_enchantment_template
item_loot_template
item_set_names
item_set_names_locale
item_template
item_template_locale
itembagfamily_dbc
itemdisplayinfo_dbc
itemextendedcost_dbc
itemlimitcategory_dbc
itemrandomproperties_dbc
itemrandomsuffix_dbc
itemset_dbc
lfg_dungeon_rewards
lfg_dungeon_template
lfgdungeons_dbc
light_dbc
linked_respawn
liquidtype_dbc
lock_dbc
mail_level_reward
mail_loot_template
mailtemplate_dbc
map_dbc
mapdifficulty_dbc
milling_loot_template
movie_dbc
namesprofanity_dbc
namesreserved_dbc
npc_spellclick_spells
npc_text
npc_text_locale
npc_trainer
npc_vendor
outdoorpvp_template
overridespelldata_dbc
page_text
page_text_locale
pet_levelstats
pet_name_generation
pet_name_generation_locale
pickpocketing_loot_template
player_class_stats
player_classlevelstats
player_factionchange_achievement
player_factionchange_items
player_factionchange_quests
player_factionchange_reputations
player_factionchange_spells
player_factionchange_titles
player_loot_template
player_race_stats
player_xp_for_level
playercreateinfo
playercreateinfo_action
playercreateinfo_cast_spell
playercreateinfo_item
playercreateinfo_skills
playercreateinfo_spell_custom
points_of_interest
points_of_interest_locale
pool_creature
pool_gameobject
pool_pool
pool_quest
pool_template
powerdisplay_dbc
prospecting_loot_template
pvpdifficulty_dbc
quest_details
quest_greeting
quest_greeting_locale
quest_mail_sender
quest_money_reward
quest_offer_reward
quest_offer_reward_locale
quest_poi
quest_poi_points
quest_request_items
quest_request_items_locale
quest_template
quest_template_addon
quest_template_locale
questfactionreward_dbc
questsort_dbc
questxp_dbc
randproppoints_dbc
reference_loot_template
reputation_reward_rate
reputation_spillover_template
scalingstatdistribution_dbc
scalingstatvalues_dbc
script_waypoint
skill_discovery_template
skill_extra_item_template
skill_fishing_base_level
skill_perfect_item_template
skillline_dbc
skilllineability_dbc
skillraceclassinfo_dbc
skilltiers_dbc
skinning_loot_template
smart_scripts
soundentries_dbc
spell_area
spell_bonus_data
spell_cooldown_overrides
spell_custom_attr
spell_dbc
spell_enchant_proc_data
spell_group
spell_group_stack_rules
spell_linked_spell
spell_loot_template
spell_mixology
spell_pet_auras
spell_proc
spell_proc_event
spell_ranks
spell_required
spell_script_names
spell_scripts
spell_target_position
spell_threat
spellcasttimes_dbc
spellcategory_dbc
spelldifficulty_dbc
spellduration_dbc
spellfocusobject_dbc
spellitemenchantment_dbc
spellitemenchantmentcondition_dbc
spellradius_dbc
spellrange_dbc
spellrunecost_dbc
spellshapeshiftform_dbc
spellvisual_dbc
stableslotprices_dbc
summonproperties_dbc
talent_dbc
talenttab_dbc
taxinodes_dbc
taxipath_dbc
taxipathnode_dbc
teamcontributionpoints_dbc
totemcategory_dbc
transportanimation_dbc
transportrotation_dbc
transports
updates
updates_include
vehicle_accessory
vehicle_dbc
vehicle_template_accessory
vehicleseat_dbc
version
warden_checks
waypoint_data
waypoint_scripts
waypoints
wmoareatable_dbc
worldmaparea_dbc
worldmapoverlay_dbc
"""

def to_lines(
    database: str,
    table: str,
):
    lines = [
        "    @cached_property",
        f"    def t_{table}(self) -> sa.Table:",
        f"        return self._metadata.tables[\"{database}.{table}\"]",
        "",
    ]
    return lines

lines = list()

for table in auth_tables.strip().splitlines():
    lines.extend(to_lines("acore_auth", table))
for table in char_tables.strip().splitlines():
    lines.extend(to_lines("acore_characters", table))
for table in world_tables.strip().splitlines():
    lines.extend(to_lines("acore_world", table))

path_output = Path.dir_here(__file__) / "orm_code.py"
path_output.write_text("\n".join(lines))
